@@ -13,6 +13,7 @@ from ..general import _pkg_root
 from scipy.constants import m_p
 from scipy.constants import e as qe
 from scipy.constants import c as clight
+from scipy.constants import epsilon_0
 
 from xobjects import BypassLinked
 
@@ -164,81 +165,82 @@ class Particles(xo.HybridClass):
         if '_xobject' in kwargs.keys():
             # Initialize xobject
             self.xoinitialize(**kwargs)
-        else:
-            if any([nn in kwargs.keys() for tt, nn in per_particle_vars]):
-                # Needed to generate consistent longitudinal variables
-                pyparticles = Pyparticles(**kwargs)
-                if 'mass_ratio' in kwargs.keys():
-                    del(kwargs['mass_ratio']) # info transferred in pyparticles.chi
+            return
 
-                part_dict = _pyparticles_to_xpart_dict(pyparticles)
-                if ('_capacity' in kwargs.keys() and
-                         kwargs['_capacity'] is not None):
-                    assert kwargs['_capacity'] >= part_dict['_num_particles']
-                else:
-                    kwargs['_capacity'] = part_dict['_num_particles']
+        if any([nn in kwargs.keys() for tt, nn in per_particle_vars]):
+            # Needed to generate consistent longitudinal variables
+            pyparticles = Pyparticles(**kwargs)
+            if 'mass_ratio' in kwargs.keys():
+                del(kwargs['mass_ratio']) # info transferred in pyparticles.chi
+
+            part_dict = _pyparticles_to_xpart_dict(pyparticles)
+            if ('_capacity' in kwargs.keys() and
+                        kwargs['_capacity'] is not None):
+                assert kwargs['_capacity'] >= part_dict['_num_particles']
             else:
-                pyparticles = None
-                if '_capacity' not in kwargs.keys():
-                    kwargs['_capacity'] = 1
+                kwargs['_capacity'] = part_dict['_num_particles']
+        else:
+            pyparticles = None
+            if '_capacity' not in kwargs.keys():
+                kwargs['_capacity'] = 1
 
-            # Make sure _capacity is integer
-            kwargs['_capacity'] = int(kwargs['_capacity'])
+        # Make sure _capacity is integer
+        kwargs['_capacity'] = int(kwargs['_capacity'])
 
-            # We just provide array sizes to xoinitialize (we will set values later)
-            kwargs.update(
-                    {kk: kwargs['_capacity'] for tt, kk in per_particle_vars})
+        # We just provide array sizes to xoinitialize (we will set values later)
+        kwargs.update(
+                {kk: kwargs['_capacity'] for tt, kk in per_particle_vars})
 
-            if 'pzeta' in kwargs.keys():
-                del(kwargs['pzeta'])  # handled in part_dict
+        if 'pzeta' in kwargs.keys():
+            del(kwargs['pzeta'])  # handled in part_dict
 
-            if 'sigma' in kwargs.keys():
-                raise NameError(
-                    '`sigma` is not supported anymore. Please use `zeta` instead.')
+        if 'sigma' in kwargs.keys():
+            raise NameError(
+                '`sigma` is not supported anymore. Please use `zeta` instead.')
 
-            if 'psigma' in kwargs.keys():
-                raise NameError(
-                    '`psigma` is not supported anymore. Please use `pzeta` instead.')
+        if 'psigma' in kwargs.keys():
+            raise NameError(
+                '`psigma` is not supported anymore. Please use `pzeta` instead.')
 
-            # Initialize xobject
-            self.xoinitialize(**kwargs)
+        # Initialize xobject
+        self.xoinitialize(**kwargs)
 
-            if 'start_tracking_at_element' not in kwargs.keys():
-                self.start_tracking_at_element = -1
+        if 'start_tracking_at_element' not in kwargs.keys():
+            self.start_tracking_at_element = -1
 
-            # Initialize coordinates
-            with self._bypass_linked_vars():
-                if pyparticles is not None:
-                    context = self._buffer.context
-                    for tt, kk in list(scalar_vars):
-                        setattr(self, kk, part_dict[kk])
-                    for tt, kk in list(per_particle_vars):
-                        if kk.startswith('_rng'):
-                            getattr(self, kk)[:] = 0
-                            continue
-                        vv = getattr(self, kk)
-                        vals =  context.nparray_to_context_array(part_dict[kk])
-                        ll = len(vals)
-                        vv[:ll] = vals
-                        vv[ll:] = LAST_INVALID_STATE
-                else:
-                    for tt, kk in list(scalar_vars):
-                        setattr(self, kk, 0.)
+        # Initialize coordinates
+        with self._bypass_linked_vars():
+            if pyparticles is not None:
+                context = self._buffer.context
+                for tt, kk in list(scalar_vars):
+                    setattr(self, kk, part_dict[kk])
+                for tt, kk in list(per_particle_vars):
+                    if kk.startswith('_rng'):
+                        getattr(self, kk)[:] = 0
+                        continue
+                    vv = getattr(self, kk)
+                    vals =  context.nparray_to_context_array(part_dict[kk])
+                    ll = len(vals)
+                    vv[:ll] = vals
+                    vv[ll:] = LAST_INVALID_STATE
+            else:
+                for tt, kk in list(scalar_vars):
+                    setattr(self, kk, 0.)
 
-                    for tt, kk in list(per_particle_vars):
-                        if kk == 'chi' or kk == 'charge_ratio' or kk == 'state':
-                            value = 1.
-                        elif kk == 'particle_id':
-                            value = np.arange(0, self._capacity, dtype=np.int64)
-                        else:
-                            value = 0.
-                        getattr(self, kk)[:] = value
+                for tt, kk in list(per_particle_vars):
+                    if kk == 'chi' or kk == 'charge_ratio' or kk == 'state':
+                        value = 1.
+                    elif kk == 'particle_id':
+                        value = np.arange(0, self._capacity, dtype=np.int64)
+                    else:
+                        value = 0.
+                    getattr(self, kk)[:] = value
 
             self._num_active_particles = -1  # To be filled in only on CPU
             self._num_lost_particles = -1  # To be filled in only on CPU
 
             # Force values provided by user if compatible
-            for nn in part_energy_varnames():
+            for nn in part_energy_varnames() + ['p0c', 'beta0', 'gamma0']:
                 vvv = self._buffer.context.nparray_from_context_array(getattr(self, nn))
                 if nn in input_kwargs.keys():
                     if hasattr(input_kwargs[nn], '__len__'):
@@ -247,18 +249,18 @@ class Particles(xo.HybridClass):
                         ll = len(vvv)
 
                     if np.isscalar(input_kwargs[nn]):
-                        getattr(self, "_"+nn)[:] = input_kwargs[nn]
+                        getattr(self, nn)[:] = input_kwargs[nn]
                     else:
-                        getattr(self, "_"+nn)[:ll] = (
+                        getattr(self, nn)[:ll] = (
                                 context.nparray_to_context_array(
                                     np.array(input_kwargs[nn])))
 
-            if isinstance(self._buffer.context, xo.ContextCpu):
-                # Particles always need to be organized to run on CPU
-                if '_no_reorganize' in kwargs.keys() and kwargs['_no_reorganize']:
-                    pass
-                else:
-                    self.reorganize()
+        if isinstance(self._buffer.context, xo.ContextCpu):
+            # Particles always need to be organized to run on CPU
+            if '_no_reorganize' in kwargs.keys() and kwargs['_no_reorganize']:
+                pass
+            else:
+                self.reorganize()
 
     def init_pipeline(self, name):
         self.name = name
@@ -325,6 +327,28 @@ class Particles(xo.HybridClass):
         import pandas as pd
         return pd.DataFrame(dct)
 
+    def show(self):
+        """
+        Method to print particle properties
+        """
+        df = self.to_pandas()
+        dash = '-' * 55
+        print("PARTICLES:\n\n")
+        print('{:<27} {:>12}'.format("Property", "Value"))
+        print(dash)
+        for column in df:
+            print('{:<27} {:>12}'.format(df[column].name, df[column].values[0]))
+        print(dash)
+        print('\n')
+
+    def get_classical_particle_radius0(self):
+        """
+        Method to calculate classical particle radius from reference particle
+        """
+        m0 = self.mass0*qe/(clight**2) # electron volt - kg conversion
+        r0 = (self.q0*qe)**2/(4*np.pi*epsilon_0*m0*clight**2)  #1.5347e-18 is default for protons
+        return r0
+
     @classmethod
     def from_pandas(cls, df, _context=None, _buffer=None, _offset=None):
         dct = df.to_dict(orient='list')
@@ -479,10 +503,13 @@ class Particles(xo.HybridClass):
     def hide_lost_particles(self, _assume_reorganized=False):
         self._lim_arrays_name = '_num_active_particles'
         if not _assume_reorganized:
-            self.reorganize()
+            n_active, _ = self.reorganize()
+            self._num_active_particles = n_active
 
     def unhide_lost_particles(self):
         del(self._lim_arrays_name)
+        if not isinstance(self._context, xo.ContextCpu):
+            self._num_active_particles = -1
 
     @property
     def lost_particles_are_hidden(self):
@@ -535,18 +562,20 @@ class Particles(xo.HybridClass):
         mask_active = self.state > 0
         mask_lost = (self.state < 1) & (self.state>LAST_INVALID_STATE)
 
-        n_active = np.sum(mask_active)
-        n_lost = np.sum(mask_lost)
+        n_active = int(np.sum(mask_active))
+        n_lost = int(np.sum(mask_lost))
 
-        with self._bypass_linked_vars():
-            for tt, nn in self._structure['per_particle_vars']:
-                vv = getattr(self, nn)
-                vv_active = vv[mask_active]
-                vv_lost = vv[mask_lost]
+        if not mask_active[:n_active].all():
+            # Reorganize particles
+            with self._bypass_linked_vars():
+                for tt, nn in self._structure['per_particle_vars']:
+                    vv = getattr(self, nn)
+                    vv_active = vv[mask_active]
+                    vv_lost = vv[mask_lost]
 
-                vv[:n_active] = vv_active
-                vv[n_active:n_active+n_lost] = vv_lost
-                vv[n_active+n_lost:] = LAST_INVALID_STATE
+                    vv[:n_active] = vv_active
+                    vv[n_active:n_active+n_lost] = vv_lost
+                    vv[n_active+n_lost:] = LAST_INVALID_STATE
 
         if isinstance(self._buffer.context, xo.ContextCpu):
             self._num_active_particles = n_active
